/**
 * OW2 Util
 * Copyright (C) 2009 Bull S.A.S.
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 *
 * --------------------------------------------------------------------------
 * $Id: ActionScheduler.java 5208 2009-10-11 21:48:01Z gaellalire $
 * --------------------------------------------------------------------------
 */

package org.ow2.util.pool.impl.enhanced.internal.actionscheduler;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.Executor;
import java.util.concurrent.Future;

import org.ow2.util.log.Log;
import org.ow2.util.log.LogFactory;


/**
 *
 * @author Gael Lalire
 */
public class ActionScheduler implements IActionScheduler {

    private static final Log LOG = LogFactory.getLog(ActionScheduler.class);

    private static final int ELEMENT_TO_KEEP_FOR_AVERAGE_COMPUTATION = 20;

    private LinkedList<Future<Boolean>> scheduledCallableList;

    private IAction action;

    private Executor executor;

    private Object mutex = new Object();

    private Average average = new Average(ELEMENT_TO_KEEP_FOR_AVERAGE_COMPUTATION);

    public ActionScheduler(final IAction action, final Executor executor) {
        this.executor = executor;
        this.action = action;
        scheduledCallableList = new LinkedList<Future<Boolean>>();
    }

    public void scheduleAction(final int time) {
        for (int i = 0; i < time; i++) {
            FutureTaskReallyCancellable<Boolean> futureTask = new FutureTaskReallyCancellable<Boolean>(action) {

                private long startTime;

                @Override
                public void run() {
                    startTime = System.currentTimeMillis();
                    super.run();
                }

                @Override
                protected void done() {
                    if (!isCancelled()) {
                        synchronized (mutex) {
                            scheduledCallableList.remove(this);
                        }
                        try {
                            if (get().booleanValue()) {
                                average.addElement(System.currentTimeMillis() - startTime);
                            }
                        } catch (InterruptedException e) {
                            LOG.error("No InterruptedException on done task expected", e);
                        } catch (ExecutionException e) {
                            LOG.error("Task failed", e);
                        }
                    }
                }

            };
            synchronized (mutex) {
                scheduledCallableList.addFirst(futureTask);
            }
            executor.execute(futureTask);
        }
    }

    public int cancelAction(final int time) {
        synchronized (mutex) {
            int canceledSheduledCallable = 0;
            Iterator<Future<Boolean>> iterator = scheduledCallableList.iterator();
            while (iterator.hasNext() && canceledSheduledCallable < time) {
                Future<Boolean> future = iterator.next();
                if (future.cancel(false)) {
                    canceledSheduledCallable++;
                    iterator.remove();
                }
            }
            return canceledSheduledCallable;
        }
    }

}
